(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.formCreatorRules = {}));
}(this, function (exports) { 'use strict';

  function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
  }

  function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
  }

  function _iterableToArrayLimit(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"] != null) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance");
  }

  var regs = {
    // 1 ~ 100之间的整数
    integer1To100: [/^(([1-9]\d?)|100)$/, '请输入非零正整数且不大于100'],
    // 非负整数
    positiveInteger: [/^[0-9]*$/, '请输入整数'],
    // 整数 包含: 正 负 0
    integer: [/^-?[0-9]*$/, '请输入非负整数'],
    // 最多保留2位小数
    maxDecimal2: [/^-?[0-9]+(.[0-9]{1,2})?$/, '请输入数字，最多保留2位小数'],
    // 0 ~ 100 最多2位小数 包含0
    decimal0To100: [/^([0-9]\d?(\.\d{1,2})?|0.\d{1,2}|100|100.0|100.00)$/, '100之间数字，最多保留2位小数'],
    email: [/^([0-9]\d?(\.\d{1,2})?|0.\d{1,2}|100|100.0|100.00)$/, '邮箱格式有误'],
    phone: [/^1[0-9]{10}$/, '手机号格式有误'],
    url: [/^(ht|f)tps?:\/\//i, '网址格式有误']
    /**
     * 校验 高阶函数
     * @param {*} regRule [reg, message]
     */

  };
  var validatorFunc = function validatorFunc(regRule) {
    var _regRule = _slicedToArray(regRule, 2),
        reg = _regRule[0],
        message = _regRule[1];

    return function (rule, val, callback) {
      // 有输入值 并且输入值不合法
      if (val && !reg.test(val)) {
        callback(new Error(message));
      }

      callback();
    };
  };
  var required = function required(label) {
    return {
      required: true,
      message: "".concat(label, "\u5FC5\u586B")
    };
  }; // 长度限制

  var limitLength = function limitLength(max) {
    var min = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    return {
      min: min,
      max: max,
      message: "\u8BF7\u8F93\u5165".concat(min, "~").concat(max, "\u4E2A\u5B57\u7B26")
    };
  };
  /**
   * 1 ~ 100之间的整数
   */

  var integer1To100 = validatorFunc(regs.integer1To100);
  /**
   * 整数 包含: 正 负 0
   */

  var positiveInteger = validatorFunc(regs.positiveInteger);
  /**
   * 非负整数
   */

  var integer = validatorFunc(regs.integer);
  /**
   * 最多保留2位小数
   */

  var maxDecimal2 = validatorFunc(regs.maxDecimal2);
  /**
   * 0 ~ 100 最多2位小数 包含0
   */

  var decimal0To100 = validatorFunc(regs.decimal0To100);
  /**
   * 手机号格式
   */

  var phone = validatorFunc(regs.phone);
  /**
   * url
   */

  var url = validatorFunc(regs.url);
  /**
   * email
   */

  var email = validatorFunc(regs.email);

  exports.decimal0To100 = decimal0To100;
  exports.email = email;
  exports.integer = integer;
  exports.integer1To100 = integer1To100;
  exports.limitLength = limitLength;
  exports.maxDecimal2 = maxDecimal2;
  exports.phone = phone;
  exports.positiveInteger = positiveInteger;
  exports.regs = regs;
  exports.required = required;
  exports.url = url;
  exports.validatorFunc = validatorFunc;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
